@@ -1,11 +1,9 @@
-# WashPost.pm
-# by Martin Thurn
-# Copyright (C) 1996-1998 by USC/ISI
-# $Id: WashPost.pm,v 2.78 2007/05/19 23:49:38 Daddy Exp $
+
+# $Id: WashPost.pm,v 2.80 2008/05/06 02:55:24 Martin Exp $
 
 =head1 NAME
 
-WWW::Search::WashPost - backend for searching www.washingtonpost.com
+WWW::Search::WashPost - WWW::Search backend for searching www.washingtonpost.com
 
 =head1 SYNOPSIS
 
@@ -25,6 +23,31 @@ F<http://www.washingtonpost.com>.
 This class exports no public interface; all interaction should
 be done through L<WWW::Search> objects.
 
+=head1 NOTES
+
+In the resulting WWW::SearchResult objects,
+the following elements will be set (in addition to the usual title, description, etc.):
+
+=over
+
+=item source
+
+Something like "Post" or a certain "Edition".
+
+=item location
+
+The page number of the print edition, e.g. "E05".
+
+=item category
+
+The type of item found, either "Article" or "Photo".
+
+=item seller
+
+The author or photographer.
+
+=back
+
 =head1 SEE ALSO
 
 To make new backends, see L<WWW::Search>.
@@ -33,6 +56,7 @@ To make new backends, see L<WWW::Search>.
 
 This backend (and all its subclasses) only searches news stories from
 the last 14 days.
+(This is a restriction on the Washington Post website that we can not avoid.)
 
 =head1 BUGS
 
@@ -40,38 +64,27 @@ Please tell the author if you find any!
 
 =head1 AUTHOR
 
-C<WWW::Search::WashPost> is maintained by Martin Thurn
-(mthurn@cpan.org).
+Martin Thurn <mthurn@cpan.org>
 
-=head1 LEGALESE
+=head1 LICENSE
 
-THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
-WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-
-=head1 VERSION HISTORY
-
-see ChangeLog
+This software is released under the same license as Perl itself.
 
 =cut
 
-#####################################################################
-
 package WWW::Search::WashPost;
 
 use strict;
-use vars qw( @ISA $VERSION $MAINTAINER );
 
-@ISA = qw( WWW::Search );
-
-$VERSION = do { my @r = (q$Revision: 2.78 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
-$MAINTAINER = 'Martin Thurn <mthurn@cpan.org>';
-
-use WWW::Search;
-use WWW::SearchResult;
 use URI;
+use base 'WWW::Search';
+use WWW::SearchResult;
 
-sub native_setup_search
+our
+$VERSION = do { my @r = (q$Revision: 2.80 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
+our $MAINTAINER = 'Martin Thurn <mthurn@cpan.org>';
+
+sub _native_setup_search
   {
   my ($self, $sQuery, $native_options_ref) = @_;
 
@@ -117,48 +130,47 @@ sub native_setup_search
 
   # Finally, figure out the url.
   $self->{_next_url} = $self->{_options}{'search_url'} .'?'. $self->hash_to_cgi_string($options_ref);
-  } # native_setup_search
+  } # _native_setup_search
 
-sub preprocess_results_page_OFF
+sub _preprocess_results_page_OFF
   {
   my $self = shift;
   my $sPage = shift;
   print STDERR $sPage if (2 < $self->{_debug});
   return $sPage;
-  } # preprocess_results_page
+  } # _preprocess_results_page
 
 my $WS = q{[\t\r\n\240\ ]};
 
-sub parse_tree
+sub _parse_tree
   {
   my $self = shift;
   my $oTree = shift;
   my $hits_found = 0;
-  if (! $self->approximate_hit_count)
+  # Look for the total hit count:
+  my @aoSPANcount = $oTree->look_down(
+                                      _tag => 'div',
+                                      class => 'redBlock',
+                                     );
+ COUNT_SPAN_TAG:
+  foreach my $oSPAN (@aoSPANcount)
     {
-    # Look for the total hit count:
-    my $oSPANcount = $oTree->look_down(
-                                       _tag => 'span',
-                                       id => 'currentResults',
-                                      );
-    if (ref $oSPANcount)
+    if (ref $oSPAN)
       {
-      my $oSPAN = $oSPANcount->parent;
-      if (ref $oSPAN)
+      my $sSPAN = $oSPAN->as_text;
+      print STDERR " +   try SPANcount == $sSPAN\n" if 2 <= $self->{_debug};
+      if ($sSPAN =~ m!([0-9,]+)$WS+results!i)
         {
-        my $sSPAN = $oSPAN->as_text;
-        print STDERR " +   try SPANcount == $sSPAN\n" if 2 <= $self->{_debug};
-        if ($sSPAN =~ m!$WS of$WS+([0-9,]+)$WS+results!ix)
-          {
-          my $sCount = $1;
-          print STDERR " +     raw    count == $sCount\n" if 2 <= $self->{_debug};
-          $sCount =~ s!,!!g;
-          # print STDERR " +     cooked count == $sCount\n" if 2 <= $self->{_debug};
-          $self->approximate_result_count($sCount);
-          } # if
+        my $sCount = $1;
+        print STDERR " +     raw    count == $sCount\n" if 2 <= $self->{_debug};
+        $sCount =~ s!,!!g;
+        # print STDERR " +     cooked count == $sCount\n" if 2 <= $self->{_debug};
+        $self->approximate_result_count($sCount);
+        last COUNT_SPAN_TAG;
         } # if
       } # if
-    } # if need hit count
+    } # foreach COUNT_SPAN_TAG
+  # $oTree->objectify_text;
 
   # Find all the results:
   my @aoSPAN = $oTree->look_down(
@@ -170,99 +182,94 @@ sub parse_tree
     {
     next SPAN_TAG unless ref $oSPAN;
     print STDERR " +   try oSPAN ===", $oSPAN->as_HTML, "===\n" if (2 <= $self->{_debug});
-    my $oDIVheadline = $oSPAN->look_down('_tag' => 'div',
-                                         class => 'resultDisplay',
+    my $oDIVheadline = $oSPAN->look_down('_tag' => 'h2',
                                         );
     next SPAN_TAG unless ref $oDIVheadline;
     my $oA = $oDIVheadline->look_down('_tag' => 'a');
     next SPAN_TAG unless ref $oA;
     my $sURL = $oA->attr('href');
-    my $sTitle = &strip($oA->as_text);
+    my $sTitle = _strip($oA->as_text);
     print STDERR " +     found <A>, url=$sURL=\n" if (2 <= $self->{_debug});
     print STDERR " +              title=$sTitle=\n" if (2 <= $self->{_debug});
     my $hit = new WWW::SearchResult;
     $hit->add_url($sURL);
     $hit->title($sTitle);
-    $oA->detach;
-    $oA->delete;
     my $oDIVdate = $oSPAN->look_down('_tag' => 'p',
-                                     class => 'kicker',
-                                    );
+                                     class => 'kicker');
     if (ref($oDIVdate))
       {
       my $s = $oDIVdate->as_text;
-      $s =~ s!\A.+\|\s+!!;
-      $hit->change_date($s);
+      print STDERR " DDD   raw date (kicker) =$s=\n" if 2 <= $self->{_debug};
+      if ($s =~ m/(.+)\ \|\ (.+)/x)
+        {
+        my $sType = $1;
+        $hit->category($sType);
+        my $sDate = $2;
+        $hit->change_date($sDate);
+        } # if
       } # if
-    $oDIVdate->detach;
-    $oDIVdate->delete;
     my $oDIVdesc = $oSPAN->look_down('_tag' => 'p',
-                                     class => 'teaser',
-                                    );
+                                     class => 'teaser');
     if (ref($oDIVdesc))
       {
       $hit->description($oDIVdesc->as_text);
       } # if
-    $oDIVdesc->detach;
-    $oDIVdesc->delete;
-    my $s = $oSPAN->as_text;
-    if ($s =~ m!\((.+)\)!)
+    my $oDIVbyline = $oSPAN->look_down('_tag' => 'p',
+                                       class => 'stamp',
+                                      );
+    if (ref($oDIVbyline))
       {
-      $hit->source($1);
+      my $s = $oDIVbyline->as_text;
+      if ($s =~ m!\((.+)\)!)
+        {
+        $hit->source($1);
+        } # if
+      if ($s =~ m/\A(.+), (\S+)/)
+        {
+        $hit->seller($1);
+        $hit->location($2);
+        }
       } # if
     push(@{$self->{cache}}, $hit);
     $self->{'_num_hits'}++;
     $hits_found++;
     } # foreach SPAN_TAG
 
-  # This is the next-page URL:
-  # http://www.washingtonpost.com/ac2/wp-dyn/NewsSearch?st=treasure&fn=&sfn=&sa=np&cp=2&hl=false&sb=-1&sd=&ed=&blt=&fa_1_sourcenavigator="The+Washington+Post"
-  # The page uses JavaScript to fill-in and submit the form.  In order
-  # to do it mechanically, we need to keep track of what page we're on
-  # and put that number in cp, along with sa=np
+  # The next-page link uses JavaScript to fill-in and submit the form.
+  # In order to do it mechanically, we need to keep track of what page
+  # we're on and put that number in cp, along with sa=np
 
   # Find the next link, if any:
-  my $oNext = $oTree->look_down('_tag', 'div',
-                                class => 'pagination',
-                               );
-  if (ref($oNext))
-    {
-    my $s = $oNext->as_HTML;
-    print STDERR " DDD oNext is =$s=\n" if (2 <= $self->{_debug});
-    my @aoAnext = $oNext->look_down(_tag => 'a');
-    my $oAnext = pop @aoAnext;
-    if (ref($oAnext))
-      {
-      my $s = $oAnext->as_HTML;
-      print STDERR " DDD   try oANext is =$s=\n" if (2 <= $self->{_debug});
-      # Sanity check:
-      if ($oAnext->as_text eq 'Next>')
-        {
-        $self->{_options}->{sa} = 'np';
-        $self->{_options}->{cp} = ++$self->{_washpost_cp};
-        $self->{_next_url} = $self->{_options}{'search_url'} .'?'. $self->hash_to_cgi_string($self->{_options});
-        } # if
-      } # if
-    } # if
+  my $oDiv = $oTree->look_down('_tag', 'div',
+                               class => 'pagination',
+                              );
+  goto SKIP_NEXT_LINK if (! ref $oDiv);
+  my @aoAnext = $oDiv->look_down(_tag => 'a');
+  my $oAnext = pop @aoAnext;
+  goto SKIP_NEXT_LINK if (! ref($oAnext));
+  goto SKIP_NEXT_LINK if ($oAnext->as_text ne 'Next>');
+  $self->{_options}->{sa} = 'np';
+  $self->{_options}->{cp} = ++$self->{_washpost_cp};
+  $self->{_next_url} = $self->{_options}{'search_url'} .'?'. $self->hash_to_cgi_string($self->{_options});
  SKIP_NEXT_LINK:
   return $hits_found;
-  } # parse_tree
+  } # _parse_tree
 
 
-sub strip
+sub _strip
   {
   my $s = shift;
   $s =~ s!\A$WS+!!x;
   $s =~ s!$WS+\Z!!x;
   return $s;
-  } # strip
+  } # _strip
 
 
-sub octalize
+sub _octalize
   {
   my $s = shift;
   return sprintf "\\%.3lo" x length($s), unpack("C*", $s);
-  } # octalize
+  } # _octalize
 
 1;
 
